#!/usr/bin/env python

import argparse
import csv
from datetime import datetime
import json
import os
import openpyxl
from openpyxl.utils import get_column_letter
import sys


class ReportType:
	ACCESS_REPORT = 0
	CLOCKING_REPORT = 1

def to_str(input):
	'''
	(bytes == str) in python2
	(bytes != str) in python3
	'''
	if str == type(input):
		return str(input)
	elif bytes == type(input):
		# for python3
		return str(input, 'utf-8')
	else:
		return str(input)

def process_input_data(src_file):
	with open(src_file) as f:
		input_data = json.load(f)

	report_type = input_data['report_type']
	multilang_strings = input_data['strings']

	column_names = ['date', 'time', 'task', 'account', 'name', 'group', 'event', 'similarity']
	records = []
	for captured_face in input_data['data']['captured_face']:
		start_time = captured_face.get('start_time', 0.) / 1000.0
		datetime_obj = datetime.fromtimestamp(start_time)
		date = datetime_obj.strftime('%Y-%m-%d')
		time = datetime_obj.strftime('%H:%M:%S')

		group_names = '-'
		list_group_names = captured_face.get('registered_face_group_names', [])
		if len(list_group_names) > 0:
			group_names = ', '.join(list_group_names)

		similarity = captured_face.get('similarity', 0)
		if similarity == 0:
			similarity = '-'

		records.append([
			date,
			time,
			to_str(captured_face.get('task_name', '-').encode('utf-8')),
			to_str(captured_face.get('registered_face_account', '-').encode('utf-8')),
			to_str(captured_face.get('registered_face_name', '-').encode('utf-8')),
			to_str(group_names.encode('utf-8')),
			to_str(multilang_strings.get(captured_face.get('event', '-'), '-').encode('utf-8')),
			similarity,
		])

	return records, column_names, report_type, multilang_strings


def process_for_access_report(records, column_names, multilang_strings):
	date_index = column_names.index('date')
	time_index = column_names.index('time')

	records.sort(key = lambda record: (record[date_index], record[time_index]))
	multilang_column_names = [to_str(multilang_strings[column_name].encode('utf-8')) for column_name in column_names]
	return records, multilang_column_names


def process_for_clocking_report(records, column_names, multilang_strings):
	date_index = column_names.index('date')
	time_index = column_names.index('time')
	task_index = column_names.index('task')
	account_index = column_names.index('account')
	name_index = column_names.index('name')
	group_index = column_names.index('group')

	records.sort(key = lambda record: (record[date_index], record[account_index], record[time_index]))
	clocking_column_names = ['date', 'account', 'name', 'group', 'time_first', 'task_first', 'time_last', 'task_last', 'duration']
	time_first_index = clocking_column_names.index('time_first')
	time_last_index = clocking_column_names.index('time_last')
	task_last_index = clocking_column_names.index('task_last')
	duration_index = clocking_column_names.index('duration')

	clocking_records = []
	current_account = None
	current_date = None
	for i in range(len(records)):
		record = records[i]
		if record[date_index] != current_date or record[account_index] != current_account:
			current_date = record[date_index]
			current_account = record[account_index]
			clocking_records.append([
				record[date_index],
				record[account_index],
				record[name_index],
				record[group_index],
				record[time_index],
				record[task_index],
				'-',
				'-',
				'-'
			])
			continue

		is_last_continuous_record = False
		if i + 1 < len(records):
			next_record = records[i + 1]
			next_account = next_record[account_index]
			next_date = next_record[date_index]
			if next_date != current_date or next_account != current_account:
				is_last_continuous_record = True
		elif i + 1 == len(records):
			is_last_continuous_record = True

		if is_last_continuous_record:
			clocking_records[-1][time_last_index] = record[time_index]
			clocking_records[-1][task_last_index] = record[task_index]

			time_first = clocking_records[-1][time_first_index]
			time_last = clocking_records[-1][time_last_index]
			time_delta = datetime.strptime(time_last, '%H:%M:%S') - datetime.strptime(time_first, '%H:%M:%S')
			clocking_records[-1][duration_index] = to_str(time_delta)

	multilang_column_names = [to_str(multilang_strings[column_name].encode('utf-8')) for column_name in clocking_column_names]
	return clocking_records, multilang_column_names


def save_as_xlsx(records, column_names, dest_file, sheet_title):
	workbook = openpyxl.Workbook()
	sheet = workbook.active
	sheet.title = sheet_title

	sheet.append(column_names)
	for record in records:
		sheet.append(record)

	workbook.save(filename=dest_file)


def save_as_csv(records, column_names, dest_file):
	with open(dest_file, 'w') as f:
		writer = csv.writer(f)
		writer.writerow(column_names)
		writer.writerows(records)


def parse_args():
	parser = argparse.ArgumentParser(description='Generate face report from JSON file')
	parser.add_argument('--src', dest='src_file', type=str, help='Absolute path of input file', required=True)
	parser.add_argument('--dest', dest='dest_file', type=str, help='Absolute path of output file', required=True)

	args = parser.parse_args()
	return args


if __name__ == '__main__':
	args = parse_args()

	src_file = args.src_file
	dest_file = args.dest_file

	src_ext = os.path.splitext(src_file)[1]
	dest_ext = os.path.splitext(dest_file)[1]

	if src_ext != '.json':
		print('Error: wrong input format, need to be .json file')
		sys.exit(1)

	if dest_ext != '.xlsx' and dest_ext != '.csv':
		print('Error: wrong outputput format, need to be .xlsx or .csv file')
		sys.exit(1)

	records, column_names, report_type, multilang_strings = process_input_data(src_file)

	if report_type == ReportType.ACCESS_REPORT:
		records, column_names = process_for_access_report(records, column_names, multilang_strings)
		sheet_title = multilang_strings['access_report']
	else:
		records, column_names = process_for_clocking_report(records, column_names, multilang_strings)
		sheet_title = multilang_strings['clocking_report']

	if dest_ext == '.xlsx':
		save_as_xlsx(records, column_names, dest_file, sheet_title)
	elif dest_ext == '.csv':
		save_as_csv(records, column_names, dest_file)
