#!/usr/bin/env python

from __future__ import print_function

from openpyxl import Workbook
from openpyxl.chart import BarChart, Reference, Series
from openpyxl.utils import get_column_letter
import argparse
import sys
import os.path
import json

def to_str(input):
    '''
    (bytes == str) in python2
    (bytes != str) in python3
    '''
    if str == type(input):
        return str(input)
    elif bytes == type(input):
        # for python3
        return str(input, 'utf-8')
    else:
        return str(input)

def gen_table_data(json_report, sheet):
    sheet.title = json_report['title']

    # first row: task names
    sheet['A1'] = to_str(json_report['taskNames'].encode('utf-8'))

    # second row: date string
    sheet['A2'] = json_report['date']

    # third row: column_name
    col_num = 0
    for col_name in json_report['columns']:
        col_num += 1
        sheet.cell(column=col_num, row=3, value=col_name)
        sheet.column_dimensions[get_column_letter(col_num)].width = 20

    # rest rows: selected counts
    row_num = 3
    for row in sorted(json_report['report'], key=lambda x : x['id']):
        row_num += 1
        col_num = 0
        for field in json_report['fields']:
            col_num += 1
            sheet.cell(column=col_num, row=row_num, value=row[field])

    return row_num

def gen_bar_chart(json_report, max_row_idx, max_col_idx, sheet):
    chart = BarChart()
    chart.type = 'col'
    chart.style = 10
    chart.title = json_report['title']
    chart.width = 25 # cm

    chartIdx = 0
    for chartColumn in json_report['chartColumns']:
        values = Reference(sheet, min_col=chartIdx + 2, min_row=4, max_row=max_row_idx)
        series = Series(values, title=chartColumn)
        chart.append(series)
        chartIdx += 1

    category = Reference(sheet, min_col=1, min_row=4, max_row=max_row_idx)

    chart.set_categories(category)
    chart.shape = 4

    idx = 0
    for color in json_report['colors']:
        chart.series[idx].graphicalProperties.line.solidFill = color
        chart.series[idx].graphicalProperties.solidFill = color
        idx += 1

    sheet.add_chart(chart, 'F4')

def get_parser():
    parser = argparse.ArgumentParser(description='Convert DVA json report to .xlsx file')
    parser.add_argument('--input', dest='input_file', type=str, help='Absolute path of input json file', required=True)
    parser.add_argument('--output', dest='output_file', type=str, help='Absolute path of output xlsx file', required=True)

    return parser

def __main__():

    parser = get_parser()
    args = parser.parse_args()

    if not args.input_file.endswith('.json') or not args.output_file.endswith('.xlsx'):
        print('Error: Wrong input/output file format.')
        parser.print_help()
        return

    if not os.path.isfile(args.input_file):
        print('Error: Input file [{0}]not existed'.format(args.input_file))
        parser.print_help()
        return

    # Read json report
    try:
        with open(args.input_file, 'r') as f:
            json_report = json.load(f)
    except ValueError:
        print('Failed to load input json file {0}'.format(args.input_file))
        return

    wb = Workbook()
    sheet = wb.active

    max_row_idx = gen_table_data(json_report, sheet)
    max_col_idx = len(json_report['columns'])
    gen_bar_chart(json_report, max_row_idx, max_col_idx, sheet)

    wb.save(filename = args.output_file)

if __name__ == '__main__':
    __main__()
