#!/bin/python3
# Version: 1 @ 2020/12/25
# Simulation: liveview * client (websocket-mixstreamer)

import time
import subprocess
import threading
import json
import os
import sys

def InstallWebsocketClient():
	print("Install pip & websocket-client.")
	subprocess.check_call([sys.executable, "-m", "ensurepip"])
	subprocess.check_call([sys.executable, "-m", "pip", "install", "websocket-client"])

try:
	import websocket
except ImportError:
	InstallWebsocketClient()
	import websocket

def CookieFile(ip):
	return 'cookie_' + ip.replace('.', '_') + '.txt'

def LoginCreateCookieFile(ip, user, passwd):
	print("Authorizing " + ip + ", " + user + " / " + passwd)
	curl_cmd = 'curl -c ' + CookieFile(ip) + ' "http://' + ip + ':5000/webman/login.cgi?username=' + user + '&passwd=' + passwd + '" > /dev/null 2>&1'
	subprocess.call(curl_cmd, shell=True)

def GetCamList(ip, offset, limit):
	print("Get camera list offset[%d], limit[%d]" %(offset, limit))
	camera_array = []

	while True:
		curl_cmd = 'curl -s -b ' + CookieFile(ip) + ' -d "api=SYNO.SurveillanceStation.Camera&method=List&version=8&ownerDsId=0&blServerChecked=true&basic=true&start=' + str(offset) + '&limit=' + str(limit) + '" "http://' + ip + ':5000/webapi/entry.cgi"'
		json_ret = json.loads(os.popen(curl_cmd).read())

		if False == json_ret['success']:
			print("Failed to get cam list [", json_ret, "], wait 5 mins.")
			sys.exit()
		else:
			for cam in json_ret['data']['cameras']:
				if cam['enabled'] == True:
					print(str(cam['id']) + ": " + cam['name'])
					camera_array.append(str(cam['id']))
			print("Total:", len(camera_array))
			break

	return camera_array

def ParseCookie(ip):
	bash_cmd = "cat " + CookieFile(ip) + " | sed -n 's/.*\\tid[\\t]*\(.*\)/\\1/p'"
	cookie_id = os.popen(bash_cmd).read()
	bash_cmd = "cat " + CookieFile(ip) + " | sed -n 's/.*\\tsmid[\\t]*\(.*\)/\\1/p'"
	cookie_smid = os.popen(bash_cmd).read()
	return 'id=' + cookie_id.rstrip('\n') + '; smid=' + cookie_smid.rstrip('\n')

def KeepaliveThread(run_event, ip):
	while(run_event.is_set()):
		time.sleep(25)
		curl_cmd = 'curl -b ' + CookieFile(ip) + ' -d "api=SYNO.SurveillanceStation.Recording&method=Keepalive&version=5" "http://' + ip + ':5000/webapi/entry.cgi" > /dev/null 2>&1'
		subprocess.call(curl_cmd, shell=True)

def CreateWebSocket(ws_path, cookie):
	def on_close(ws):
		ws.connected = False
		print(".", end =" ", flush = True)

	def on_open(ws):
		ws.connected = True

	ws = websocket.WebSocketApp(ws_path, cookie = cookie, on_open = on_open, on_close = on_close)
	ws.connected = False
	return ws

def DelayTimer(run_event, sec):
	for i in range(0, sec):
		if run_event.is_set():
			time.sleep(1)
		else:
			break;

def WsRunForever(run_event, ws):
	while(run_event.is_set()):
		ws.run_forever()
		if run_event.is_set():
			print("Websocket closed, wait 10sec.")
			DelayTimer(run_event, 10)
		if run_event.is_set():
			print("Retry", ws)

def SendKeepalive(keepalive_array):
	for ws in keepalive_array:
		if ws.connected:
			ws.send("keepAlive");

def Simulation(args):
	ip = args.ip
	thread_array = []
	ws_array = []
	keepalive_array = []
	run_event = threading.Event()
	run_event.set()
	Dir = os.path.dirname(os.path.abspath(__file__))

	try:
		LoginCreateCookieFile(ip, args.username, args.password)
		camera_array = GetCamList(ip, args.offset, args.limit)
		cookie = ParseCookie(ip)
		websocket.enableTrace(False)

		mux_arg = ''
		if True == args.raw_frame:
			mux_arg = '&blMux=true&browser=2'

		max_cam_limit = len(camera_array)
		cam_limit = args.limit_per_client
		if 0 > cam_limit:
			cam_limit = max_cam_limit

		for cnt in range(0, args.client):
			print("Client %d:" %(cnt + 1))
			print("\t+ Keepalive webapi")
			thread = threading.Thread(target = KeepaliveThread, args = (run_event, ip))
			thread_array.append(thread)

			print("\t+ Messaged")
			ws = CreateWebSocket("ws://" + ip + ":5000/ss_message_task/?ClientType=webclient", cookie)
			ws_array.append(ws)

			live_array = []
			for idx in range(0, cam_limit):
				live_array.append(camera_array[(cnt * cam_limit + idx) % max_cam_limit])

			for cam_id in live_array:
				print("\t+ MixStream, arg: id=" + cam_id + mux_arg)
				ws = CreateWebSocket("ws://" + ip + ":5000/ss_webstream_task/?method=MixStream&stmSrc=0&blLiveSharing=true&blAudio=true&pause=false&dsId=0&id=" + cam_id + mux_arg, cookie)
				ws_array.append(ws)
				keepalive_array.append(ws)

			if live_array:
				print("\t+ AlertStream, arg: camIdList=" + ','.join(live_array))
				ws = CreateWebSocket("ws://" + ip + ":5000/ss_webstream_task/?method=AlertStream&blSentFromHost=false&mode=240&dsId=0&camIdList=" + '%2C'.join(camera_array), cookie)
				ws_array.append(ws)
				keepalive_array.append(ws)

		for ws in ws_array:
			thread = threading.Thread(target = WsRunForever, args = (run_event, ws))
			thread_array.append(thread)

		print("Run thread start", end =" ", flush = True)
		for idx, thread in enumerate(thread_array):
			thread.start()
			if 9 == (idx % 10):
				print(".", end =" ", flush = True)
				time.sleep(1)
			if 99 == (idx % 100):
				SendKeepalive(keepalive_array)

		print("\nStart liveview simulation, use Ctrl + C to exit")
		while True:
			time.sleep(10)
			SendKeepalive(keepalive_array)

	except KeyboardInterrupt:
		print("\nStop liveview simulation")
		run_event.clear()

		print("Close websocket")
		for ws in ws_array:
			threading.Thread(target = ws.close).start()

		print("Wait thread stop", end =" ", flush = True)
		for thread in thread_array:
			thread.join()

		print("\nBye~")

if __name__ == "__main__":
	import argparse
	import re

	# autotune.py [-h] [-u USERNAME] [-p PASSWORD] IpPort
	parser = argparse.ArgumentParser()
	parser.add_argument("ip", help = "set ip")
	parser.add_argument("-u", "--username", default = "admin", help = "set login user name (default = admin)")
	parser.add_argument("-p", "--password", default = "123456", help = "set login user password (default = 123456)")
	parser.add_argument("-o", "--offset", default = 0, type=int, help = "set camera list offset (default = 0)")
	parser.add_argument("-l", "--limit", default = 0, type=int, help = "set camera list limit (default = 0)")
	parser.add_argument("-c", "--client", default = 1, type=int, help = "set client count (default = 1)")
	parser.add_argument("-lc", "--limit_per_client", default = -1, type=int, help = "set cam limit per client (default/negative = all camera)")
	parser.add_argument("--raw_frame", action="store_true", help = "skip mux-fmp4")
	args = parser.parse_args()

	# roughly match ip
	try:
		if None == re.match(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$', args.ip):
			raise ValueError()
	except ValueError as e:
		print(__file__ + ': error: invaild IpPort: {0}'.format(args.ip))
		sys.exit()

	Simulation(args)
	sys.exit()
